@@ -20,8 +20,8 @@ For development, I do:
 
 $ docker-shell homeassistant
 > cd /usr/local/lib/python3.7/site-packages/pyvantage/
-> cp /config/pyvantage/pyvantage/_init__.py . 
-# or 
+> cp /config/pyvantage/pyvantage/_init__.py .
+# or
 > cp /config/pyvantage/pyvantage/__init__.py /usr/local/lib/python3.7/site-packages/pyvantage
 # where /config in the docker image points to my home assistant config directory
 # which has a pyvantage subdirectory containing a clone of the github repo.
@@ -66,6 +66,10 @@ __copyright__ = "Copyright 2018, 2019, 2020 Greg J. Badros"
 # LoadGroup         | LoadGroup        | vc.outputs, vc.load_groups
 # Keypad            | Keypad           | vc.keypads
 # DualRelayStation  | Keypad           | vc.keypads
+# IRZone            | Keypad           | vc.keypads
+# Dimmer            | Keypad           | vc.keypads
+# EqCtrl            | Keypad           | vc.keypads
+# EqUx              | Keypad           | vc.keypads
 # Button            | Button           | vc.buttons
 # DryContact        | Button           | vc.buttons
 # GMem              | Variable         | vc.variables
@@ -115,6 +119,7 @@ import base64
 import re
 import json
 import os
+import traceback
 
 from collections import deque
 from xml.sax.saxutils import escape
@@ -214,7 +219,7 @@ class VantageConnection(threading.Thread):
         try:
             self._telnet[i].write(cmd.encode('ascii') + b'\r\n')
         except BrokenPipeError:
-            _LOGGER.warning("Vantage BrokenPipeError - disconnected")
+            _LOGGER.warning("Vantage BrokenPipeError - disconnected but retrying")
             self._connected[i] = False
             raise
 
@@ -314,6 +319,21 @@ def _desc_from_t1t2(title1, title2):
     return desc.strip()
 
 
+def replace_keep_case(word, replacement, text):
+    """Replace word with replacement in text.
+    While preserving the case (lower/upper/title) of word."""
+    def func(match):
+        g = match.group()
+        if g.islower():
+            return replacement.lower()
+        if g.istitle():
+            return replacement.title()
+        if g.isupper():
+            return replacement.upper()
+        return replacement
+    return re.sub(word, func, text, flags=re.I)
+
+
 class VantageXmlDbParser():
     """The parser for Vantage XML database.
 
@@ -332,6 +352,7 @@ class VantageXmlDbParser():
         self.keypads = []
         self.sensors = []
         self.load_groups = []
+        self.last_area_vid = -1
         self.vid_to_area = {}
         self.vid_to_load = {}
         self.vid_to_keypad = {}
@@ -381,6 +402,7 @@ class VantageXmlDbParser():
             area = self._parse_area(area_xml)
             _LOGGER.debug("Area = %s", area)
             self.vid_to_area[area.vid] = area
+            self.last_area_vid = area.vid
 
         irzones = root.findall(".//Objects//IRZone[@VID]")
         for irzone_xml in irzones:
@@ -390,7 +412,59 @@ class VantageXmlDbParser():
 
         loads = root.findall(".//Objects//Load[@VID]")
         loads = loads + root.findall(".//Objects//Vantage.DDGColorLoad[@VID]")
-        for load_xml in loads:
+        other_loads = []
+        color_loads = []
+        open_loads = []
+        for ld in loads:
+            t = ld.find('Name').text
+            if t.endswith(' COLOR'):
+                color_loads.append(ld)
+            elif t.lower().endswith(' open'):
+                open_loads.append(ld)
+            else:
+                other_loads.append(ld)
+        ordered_loads = open_loads + other_loads + color_loads
+        skip_load_vids = set()
+        for load_xml in ordered_loads:
+            xml_name = load_xml.find('Name').text
+            output = None
+            if xml_name.lower().endswith(" open"):
+                close_name = replace_keep_case(' open', " close", xml_name)
+                stop_name = replace_keep_case(' open', " stop", xml_name)
+                isopen_name = replace_keep_case(' open', " is open", xml_name)
+                _LOGGER.debug("Looking for close_name = %s", close_name)
+                _LOGGER.debug("Looking for stop_name = %s", stop_name)
+                _LOGGER.debug("Looking for isopen_name = %s", isopen_name)
+                close_load_xml = root.findall(
+                    ".//Objects//Load[Name='" + close_name + "']")
+                if len(close_load_xml) == 1:
+                    close_load_xml = close_load_xml[0]
+                    isopen_contact_xml = root.findall(
+                        ".//Objects//DryContact[Name='" + isopen_name + "']")
+                    if len(isopen_contact_xml) == 1:
+                        isopen_contact_xml = isopen_contact_xml[0]
+                    else:
+                        isopen_contact_xml = None
+                    stop_load_xml = root.findall(
+                        ".//Objects//Load[Name='" + stop_name + "']")
+                    if len(stop_load_xml) == 1:
+                        stop_load_xml = stop_load_xml[0]
+                    else:
+                        stop_load_xml = None
+                    shade = self._parse_3_shade(isopen_contact_xml,
+                                                load_xml,
+                                                close_load_xml,
+                                                stop_load_xml)
+                    for v in shade.vids:
+                        skip_load_vids.add(v)
+                    self.vid_to_shade[shade.vid] = shade
+                    self.outputs.append(shade)
+                    _LOGGER.debug("shade3 = %s", shade)
+                    continue
+
+            if int(load_xml.get("VID")) in skip_load_vids:
+                _LOGGER.debug("Skipping %s because used for blind3", load_xml)
+                continue
             output = self._parse_output(load_xml)
             if output is None:
                 continue
@@ -412,11 +486,16 @@ class VantageXmlDbParser():
 
         keypads = root.findall(".//Objects//Keypad[@VID]")
         keypads = keypads + root.findall(".//Objects//DualRelayStation[@VID]")
+        keypads = keypads + root.findall(".//Objects//IRZone[@VID]")
+        keypads = keypads + root.findall(".//Objects//Dimmer[@VID]")
+        keypads = keypads + root.findall(".//Objects//EqCtrl[@VID]")
+        keypads = keypads + root.findall(".//Objects//EqUX[@VID]")
         for kp_xml in keypads:
             keypad = self._parse_keypad(kp_xml)
             _LOGGER.debug("keypad = %s", keypad)
             self.vid_to_keypad[keypad.vid] = keypad
-            self.vid_to_area[keypad.area].add_keypad(keypad)
+            if keypad.area > 0:
+                self.vid_to_area[keypad.area].add_keypad(keypad)
             self.keypads.append(keypad)
 
         buttons = root.findall(".//Objects//Button[@VID]")
@@ -586,9 +665,14 @@ class VantageXmlDbParser():
         """
         try:
             vid = int(shade_xml.get('VID'))
+            area_xml = shade_xml.find('Area')
+            area_vid = self.last_area_vid
+            if area_xml is not None:
+                area_vid = int(area_xml.text)
+
             shade = Shade(self._vantage,
                           name=shade_xml.find('Name').text,
-                          area_vid=int(shade_xml.find('Area').text),
+                          area_vid=area_vid,
                           vid=vid)
             return shade
         except Exception as e:
@@ -609,7 +693,10 @@ class VantageXmlDbParser():
                 out_name = out_name.strip()
             if not out_name or out_name.isspace():
                 out_name = output_xml.find('Name').text.strip()
-            area_vid = int(output_xml.find('Area').text)
+            area_xml = output_xml.find('Area')
+            area_vid = self.last_area_vid
+            if area_xml is not None:
+                area_vid = int(area_xml.text)
 
             area_name = self.vid_to_area[area_vid].name.strip()
             lt_xml = output_xml.find('LoadType')
@@ -697,6 +784,51 @@ class VantageXmlDbParser():
         except Exception as e:
             _LOGGER.warning("Error parsing Output vid = %d: %s", vid, e)
 
+    def _parse_3_shade(self, isopen_xml, open_xml, close_xml, stop_xml):
+        """Parses three XML elements that together make a single shade.
+        open_xml is the output load low-voltage relay for opening,
+        close_xml is the output load low-voltage relaying for closing.
+        isopen_xml is the drycontact for reading whether it is open."""
+        _LOGGER.debug("_parse_3_shade io,o,c,s=%s, %s, %s, %s",
+                      isopen_xml, open_xml, close_xml, stop_xml)
+        vids = [int(isopen_xml.get('VID')) if isopen_xml else None,
+                int(open_xml.get('VID')),
+                int(close_xml.get('VID')),
+                int(stop_xml.get('VID')) if stop_xml else None]
+
+        shade_name = open_xml.find('Name').text.strip()[:-5]
+        area_xml = open_xml.find('Area')
+        area_vid = self.last_area_vid
+        if area_xml is not None:
+            area_vid = int(area_xml.text)
+
+        close_area_xml = close_xml and close_xml.find('Area')
+        close_area_vid = self.last_area_vid
+        if close_area_xml is not None:
+            close_area_vid = int(close_area_xml.text)
+
+        isopen_area_xml = isopen_xml and isopen_xml.find('Area')
+        isopen_area_vid = self.last_area_vid
+        if isopen_area_xml is not None:
+            isopen_area_vid = int(isopen_area_xml.text)
+
+        stop_area_xml = stop_xml and stop_xml.find('Area')
+        stop_area_vid = self.last_area_vid
+        if stop_area_xml is not None:
+            stop_area_vid = int(stop_area_xml.text)
+
+        if ((area_vid != close_area_vid) or
+             (isopen_xml and area_vid != isopen_area_vid) or
+             (stop_xml and area_vid != stop_area_vid)):
+            _LOGGER.warning("open/close/stop/isopen device "
+                            "areas do not match: %s", shade_name)
+            return None
+        shade = Shade3(self._vantage,
+                       name=shade_name,
+                       area_vid=area_vid,
+                       vids=vids)
+        return shade
+
     def _parse_load_group(self, output_xml):
         """Parses a load group, which is a set of loads"""
         out_name = output_xml.find('DName').text
@@ -706,7 +838,10 @@ class VantageXmlDbParser():
             out_name = output_xml.find('Name').text
         else:
             _LOGGER.debug("Using dname = %s", out_name)
-        area_vid = int(output_xml.find('Area').text)
+        area_xml = output_xml.find('Area')
+        area_vid = self.last_area_vid
+        if area_xml is not None:
+            area_vid = int(area_xml.text)
 
 #        area_name = self.vid_to_area[area_vid].name
         loads = output_xml.findall('./LoadTable/Load')
@@ -714,6 +849,7 @@ class VantageXmlDbParser():
         vid = int(vid)
 
         load_vids = []
+        color_vids = []
         dmx_color = False
         support_color_temp = False
         for load in loads:
@@ -721,10 +857,13 @@ class VantageXmlDbParser():
             load_vids.append(v)
             if self.vid_to_load[v]._dmx_color:
                 dmx_color = True
+                support_color_temp = True
+                color_vids.append(v)
                 _LOGGER.debug("for loadgroup %d, vid %s supports color",
                               vid, v)
-            if self.vid_to_load[v].support_color_temp:
+            elif self.vid_to_load[v].support_color_temp:
                 support_color_temp = True
+                color_vids.append(v)
                 _LOGGER.debug("for loadgroup %d, vid %s supports color_temp",
                               vid, v)
 
@@ -732,6 +871,7 @@ class VantageXmlDbParser():
                            name=out_name,
                            area=area_vid,
                            load_vids=load_vids,
+                           color_vids=color_vids,
                            dmx_color=dmx_color,
                            support_color_temp=support_color_temp,
                            vid=vid)
@@ -739,7 +879,8 @@ class VantageXmlDbParser():
 
     def _parse_keypad(self, keypad_xml):
         """Parses a keypad device."""
-        area_vid = int(keypad_xml.find('Area').text)
+        area_xml = keypad_xml.find('Area')
+        area_vid = int(area_xml.text) if area_xml else -1
         keypad = Keypad(self._vantage,
                         name=keypad_xml.find('Name').text + ' [K]',
                         area=area_vid,
@@ -761,10 +902,15 @@ class VantageXmlDbParser():
         # one.
         try:
             vid = int(dc_xml.get('VID'))
+            if self.vid_to_shade.get(vid):
+                _LOGGER.debug("Skipping vid=%d as drycontact "
+                              "because already part of a BLIND3", vid)
+                return None
             name = dc_xml.find('Name').text + ' [C]'
             parent = dc_xml.find('Parent')
             parent_vid = int(parent.text)
-            area_vid = int(dc_xml.find('Area').text)
+            area_xml = dc_xml.find('Area')
+            area_vid = int(area_xml.text) if area_xml else -1
             num = 0
             keypad = None
             _LOGGER.debug("Found DryContact with vid = %d", vid)
@@ -775,6 +921,7 @@ class VantageXmlDbParser():
         except Exception as e:
             _LOGGER.warning("Error parsing drycontact vid = %d: %s",
                             vid, e)
+            traceback.print_exc()
 
     def _parse_button(self, button_xml):
         """Parses a button device that part of a keypad."""
@@ -810,7 +957,7 @@ class VantageXmlDbParser():
             text2 = button_xml.find('Text2').text
             desc = _desc_from_t1t2(text1, text2)
             num = int(parent.get('Position'))
-            keypad = self._vantage._ids['KEYPAD'].get(parent_vid)
+            keypad = self.vid_to_keypad.get(parent_vid)
             if keypad is None:
                 irzone = self.vid_to_area.get(parent_vid)
                 if irzone is None:
@@ -831,6 +978,8 @@ class VantageXmlDbParser():
         except Exception as e:
             _LOGGER.warning("Error parsing button vid = %d: %s",
                             vid, e)
+            traceback.print_exc()
+
 
 
 # Connect to port 2001 and write
@@ -893,6 +1042,7 @@ class Vantage():
         self._vid_to_sensor = {}  # copied out from the parser
         self._name_to_task = {}  # copied out from the parser
         self._colorvid_to_group_vid = {}
+        self._brightnessvid_to_group_vid = {}
         self._r_cmds = ['LOGIN', 'LOAD', 'STATUS', 'GETLOAD', 'GETVARIABLE',
                         'ERROR',
                         'TASK', 'GETBLIND', 'BLIND', 'INVOKE', 'VARIABLE',
@@ -937,7 +1087,7 @@ class Vantage():
         return answer
 
     # TODO: cleanup this awful logic
-    def register_id(self, cmd_type, cmd_type2, obj):
+    def register_id(self, cmd_type, cmd_type2, obj, vid=None):
         """Registers an object (through its vid [vantage id]).
 
         This lets it receive update notifications. This is the core
@@ -946,15 +1096,17 @@ class Vantage():
 
         """
 
+        if vid == None:
+            vid = obj.vid
         # First, register the VID in our _ids map.  When we issue commands to
         # the Vantage this map lets us route the respones to the correct object
         ids = self._ids.setdefault(cmd_type, {})
         ids = self._ids.setdefault(cmd_type2, {})
-        if obj.vid in ids:
-            raise VIDExistsError("VID exists %s" % obj.vid)
-        self._ids[cmd_type][obj.vid] = obj
+        if vid in ids:
+            raise VIDExistsError("VID exists %s" % vid)
+        self._ids[cmd_type][vid] = obj
         if cmd_type2:
-            self._ids[cmd_type2][obj.vid] = obj
+            self._ids[cmd_type2][vid] = obj
 
         # Now give the object a unique name.  We prefix in reverse order the
         # areas the object is contained in.  So an object may be called "Main
@@ -1075,12 +1227,16 @@ class Vantage():
                     (typ == 'R' and
                      cmd_type in ('LOAD', 'POWER', 'CURRENT',
                                   'VARIABLE', 'SENSOR', 'LIGHT'))):
-                self.handle_update_and_notify(obj, args)
+                self.handle_update_and_notify(obj, args, vid)
 
     # Note: invoked on VantageConnection thread.
-    def handle_update_and_notify(self, obj, args):
-        """Call handle_update for the obj and for subscribers."""
-        handled = obj.handle_update(args)
+    def handle_update_and_notify(self, obj, args, vid):
+        """Call handle_update for the obj and for subscribers.
+        We have to pass the vid along, too, since there are
+        object types, e.g., Shade3, that have multiple vids
+        represented by a single object and status on any of
+        those vids goes back to the same handle_update()."""
+        handled = obj.handle_update(args, vid)
         # Now notify anyone who cares that device may have changed
         if handled and handled in self._subscribers:
             self._subscribers[handled](handled)
@@ -1379,7 +1535,7 @@ class VantageEntity:
         areas.append(self._name)
         return areas
 
-    def handle_update(self, _):
+    def handle_update(self, _, __):
         """The handle_update callback is invoked when an event is received
         for the this entity.
 
@@ -1393,8 +1549,8 @@ class VantageEntity:
 
     @property
     def kind(self):
-        """The type of object (for units in hass)."""
-        return None
+        """Returns the output type."""
+        return self._load_type
 
     @property
     def extra_info(self):
@@ -1558,7 +1714,7 @@ class Output(VantageEntity):
     # S:STATUS [vid] RGBLoad.GetColor [value]
     # S:STATUS [vid] RGBLoad.GetColorName [value]
     # INVOKE [vid] RGBLoad.SetRGBW [val0], [val1], [val2], [val3]
-    def handle_update(self, args):
+    def handle_update(self, args, _):
         """Handles an event update for this object.
         E.g. dimmer level change
 
@@ -1581,7 +1737,7 @@ class Output(VantageEntity):
                     return light
                 _LOGGER.warning("Received color change of VID %d but cannot "
                                 "find corresponding load", self._vid)
-                return self
+                return None
             _LOGGER.debug("Updating brightness %d(%s): l=%f",
                           self._vid, self._name, level)
             self._level = level
@@ -1591,6 +1747,13 @@ class Output(VantageEntity):
             if level > 0 and self._rgb_is_dirty:
                 self._invoke_rgb()
             self._query_waiters.notify()
+            bvid = self._vantage._brightnessvid_to_group_vid.get(self._vid)
+            if bvid:
+                group = self._vantage._vid_to_load[bvid]
+                _LOGGER.debug("also updating bvid %d(%s): l=%f",
+                              bvid, group._name, level)
+                group.level = level
+                group._query_waiters.notify()
         else:
             if args[0] == 'RGBLoad.GetRGB':
                 _LOGGER.info("RGBLoad.GetRGB, handling vid = %d; "
@@ -1644,15 +1807,13 @@ class Output(VantageEntity):
         """Returns true iff this load is full-color."""
         return self._dmx_color
 
-    @property
-    def level(self):
+    def _get_level(self):
         """Returns the current output level by querying the controller."""
         ev = self._query_waiters.request(self.__do_query_level)
         ev.wait(self._wait_seconds)
         return self._level
 
-    @level.setter
-    def level(self, new_level):
+    def _set_level(self, new_level):
         """Sets the new output level."""
         if self._level == new_level:
             return
@@ -1671,6 +1832,8 @@ class Output(VantageEntity):
         else:
             self._vantage.send("LOAD", self._vid, new_level)
 
+    level = property(_get_level, _set_level)
+
     @property
     def rgb(self):
         """Returns current color of the light."""
@@ -1692,10 +1855,10 @@ class Output(VantageEntity):
         self._hs = [hs_color.hsv_h, hs_color.hsv_s * 100.0]
         self._rgb = new_rgb
         self._rgb_is_dirty = True
-        self._invoke_rgb()
-        if self._rgb_is_dirty:
+        if self._level > 0:
+            self._invoke_rgb()
+        else:
             self._invoke_hs()
-            # self._rgb_is_dirty = False
 
     def _invoke_rgb(self):
         """Update the RGB of the light to self._rgb"""
@@ -1810,6 +1973,155 @@ class VantageSensor(VantageEntity):
         self._value = val
 
 
+class Shade3(VantageEntity):
+    """A shade that is made up of 3 vantage devices.
+    1) an open dry contact to initiate opening;
+    2) a close dry contact to initiate closing;
+    3) a sensor dry contact to tell if it's open.
+    An optional 4th dry-contact to stop open/close is allowed."""
+
+    CMD_TYPE = 'BTN'  # for a button -- the isopen sensor
+
+    def __init__(self, vantage, name, area_vid, vids):
+        super(Shade3, self).__init__(vantage, name, area_vid, vids[1])
+        self._is_open = None
+        self._level = None
+        self._load_type = 'BLIND3'
+        self._extra_info['vids'] = "%s" % vids
+        self._isopen_vid = vids[0]
+        self._open_vid = vids[1]
+        self._close_vid = vids[2]
+        self._stop_vid = vids[3]
+        self.vids = vids
+        if self._isopen_vid:
+            self._vantage.register_id(Shade3.CMD_TYPE, None,
+                                      self, self._isopen_vid)
+        self._vantage.register_id('LOAD', None,
+                                  self, self._open_vid)
+        self._vantage.register_id('LOAD', None,
+                                  self, self._close_vid)
+        if self._stop_vid:
+            self._vantage.register_id('LOAD', None,
+                                      self, self._stop_vid)
+        self._query_waiters = _RequestHelper()
+
+    def __str__(self):
+        """Returns a pretty-printed string for this object."""
+        return (
+            "Output3 name: '%s' area: %d type: '%s' is_open: '%s' "
+            "vids: %s" % (
+                self._name, self._area, self._load_type,
+                self._is_open, self.vids))
+
+    def __repr__(self):
+        """Returns a stringified representation of this object."""
+        return str({'name': self._name, 'vids': self.vids, 'area': self._area,
+                    'type': self._load_type, 'is_open': self._is_open})
+
+    @property
+    def simple_name(self):
+        """Return a simple pretty-printed string for this object."""
+        return 'VIDS:%s (%s) [%s]' % (self.vids, self._name, self._load_type)
+
+    def last_level(self):
+        """Returns last cached value of output level, no query is performed."""
+        return None
+
+    @property
+    def level(self):
+        """The level (i.e. position) of the shade.
+        Shade3 cannot report level, so we just return None"""
+        return self._level
+
+    @level.setter
+    def level(self, new_level):
+        if new_level == 0:
+            self.close()
+            self._level = 0
+        elif new_level == 100:
+            self.open()
+            self._level = 100
+
+    def __do_query_level(self):
+        pass
+
+    def open(self):
+        """Open the shade."""
+        ev = self._query_waiters.request(self._do_start_open)
+        ev.wait(0.5)
+        self._vantage.send("LOAD", self._open_vid, "0")
+        if not self._isopen_vid:
+            self._is_open = True
+
+    def _do_start_open(self):
+        """Issue the start open command."""
+        self._vantage.send("LOAD", self._open_vid, "100")
+
+    def stop(self):
+        """Stop the shade."""
+        if self._stop_vid:
+            ev = self._query_waiters.request(self._do_start_stop)
+            ev.wait(0.5)
+            self._vantage.send("LOAD", self._stop_vid, "0")
+        else:
+            _LOGGER.warning("stop called on blind3 with no stop stupport: %s",
+                            self)
+
+    def _do_start_stop(self):
+        """Issue the start open command."""
+        self._vantage.send("LOAD", self._stop_vid, "100")
+
+    def close(self):
+        """Stop the shade."""
+        ev = self._query_waiters.request(self._do_start_close)
+        ev.wait(0.5)
+        self._vantage.send("LOAD", self._close_vid, "0")
+        if not self._isopen_vid:
+            self._is_open = False
+
+    def _do_start_close(self):
+        """Issue the start open command."""
+        self._vantage.send("LOAD", self._close_vid, "100")
+
+    def handle_update(self, args, vid):
+        """Handle new value for shade3.
+
+        This callback is invoked by the main event loop.
+
+        """
+        if vid == self._open_vid:
+            _LOGGER.info("Got %s open_vid args = %s", self, args)
+            self._query_waiters.notify()
+            return self
+        if vid == self._close_vid:
+            _LOGGER.info("Got %s open_vid args = %s", self, args)
+            self._query_waiters.notify()
+            return self
+        if vid == self._stop_vid:
+            _LOGGER.info("Got %s open_vid args = %s", self, args)
+            self._query_waiters.notify()
+            return self
+        if vid != self._isopen_vid:
+            _LOGGER.warning("unrecognized vid %d in handle_update: %s",
+                            vid, self)
+            return None
+        if self._isopen_vid is None:
+            _LOGGER.warning("Surprised: got handle_update for Blind3 "
+                            "without isopen_vid: %s", self)
+            return None
+        value = args[0]
+
+        if value == "PRESS":
+            self._is_open = True
+        elif value == "RELEASE":
+            self._is_open = False
+        else:
+            _LOGGER.warning("Got unknown shade3 %s (%d) message: %s",
+                            self._name, self._vid, value)
+        self._query_waiters.notify()
+        return self
+
+
 class Button(VantageSensor):
     """This object represents a keypad button that we can trigger and handle
     events for (button presses)."""
@@ -1858,8 +2170,8 @@ class Button(VantageSensor):
         """Returns the VID of the keypad which contains this button."""
         return self._parent
 
-    def handle_update(self, args):
-        """Handle an event from this keypad.
+    def handle_update(self, args, _):
+        """The callback invoked by the main event loop.
 
         This callback is invoked from the VantageConnection thread.
 
@@ -1871,7 +2183,8 @@ class Button(VantageSensor):
             self._value = action
             # this transfers control to Keypad.handle_update(...)
             self._vantage.handle_update_and_notify(
-                self._keypad, [self._num, self._name, self._value])
+                self._keypad, [self._num, self._name, self._value],
+                self._vid)
         else:  # it's a drycontact
             # TODO: support per-vid flipping/control of these rewrites
             if action == 'PRESS':
@@ -1889,19 +2202,28 @@ class Button(VantageSensor):
 
 class LoadGroup(Output):
     """Represent a Vantage LoadGroup."""
-    def __init__(self, vantage, name, area, load_vids, dmx_color,
-                 support_color_temp, vid):
+    def __init__(self, vantage, name, area, load_vids, color_vids,
+                 dmx_color, support_color_temp, vid):
         """Initialize a load group"""
         super(LoadGroup, self).__init__(
             vantage, name, area, 'GROUP', 'GROUP', None, dmx_color, vid)
         self._load_vids = load_vids
+        self._color_vids = color_vids
         self._support_color_temp = support_color_temp
+        self._brightness_vid = None
+        if len(self._load_vids) == 2 and len(self._color_vids) == 1:
+            if self._load_vids[0] == self._color_vids[0]:
+                self._brightness_vid = self._load_vids[1]
+            else:
+                self._brightness_vid = self._load_vids[0]
+        if self._brightness_vid:
+            self._vantage._brightnessvid_to_group_vid[
+                self._brightness_vid] = self._vid
 
-        if not self._is_dimmable:
-            for v in load_vids:
-                if self._vantage._vid_to_load[v]._is_dimmable:
-                    self._is_dimmable = True
-                    break
+        for v in load_vids:
+            if self._vantage._vid_to_load[v]._is_dimmable:
+                self._is_dimmable = True
+                break
 
     def support_color_temp(self):
         """Returns true iff this load can be set to a color temperature."""
@@ -1910,7 +2232,7 @@ class LoadGroup(Output):
     def __str__(self):
         """Returns a pretty-printed string for this object."""
         return ("Output name: '%s' area: %d type: '%s' load: '%s' "
-                "id: %d %s%s%s%s (%s) [%s]" % (
+                "id: %d %s%s%s%s (%s) (c:%s) (b:%s) [%s]" % (
                     self._name, self._area, self._output_type,
                     self._load_type, self._vid,
                     ("(dim) " if self.is_dimmable else ""),
@@ -1918,8 +2240,31 @@ class LoadGroup(Output):
                     ("(color) " if self.support_color else ""),
                     ("(dirty) " if self._rgb_is_dirty else ""),
                     self._load_vids,
+                    self._color_vids,
+                    self._brightness_vid,
                     self.full_lineage))
 
+    def last_level(self):
+        if self._brightness_vid:
+            return self._vantage._vid_to_load.get(self._brightness_vid)._level
+        else:
+            return self._level
+
+    def _get_level(self):
+        """Returns the output level of the group.
+        Iff there is one non-color and one color load, then delegate to the non-color load."""
+        if self._brightness_vid:
+            return self._vantage._vid_to_load.get(self._brightness_vid).level
+        else:
+            return super(LoadGroup, self)._get_level()
+
+    def _set_level(self, new_level):
+        if self._brightness_vid:
+            self._vantage._vid_to_load.get(self._brightness_vid).level = new_level
+        super(LoadGroup, self)._set_level(new_level)
+
+    level = property(_get_level, _set_level)
+
     # Load Groups do not respond to RGBLoad.SetRGBW invocations
     # so we need to call them for each of the member groups that do
     def _invoke_rgb(self):
@@ -1935,6 +2280,16 @@ class LoadGroup(Output):
         if self._level > 0:
             self._rgb_is_dirty = False
 
+    def _invoke_hs(self):
+        """Update the RGB of the load group to self._rgb"""
+        (h, s) = self._hs
+        for vid in self._load_vids:
+            load = self._vantage._vid_to_load.get(vid)
+            if load and (load._dmx_color or load._load_type == "DW"):
+                self._vantage.send("INVOKE", vid,
+                                   ("RGBLoad.SetHSL %d %d %d" %
+                                    (h, s, self._level)))
+
     def __do_query_level(self):
         """Helper to perform the actual query the current dimmer level of the
         output. For pure on/off loads the result is either 0.0 or 100.0."""
@@ -1986,7 +2341,7 @@ class Keypad(VantageSensor):
         """The type of object (for units in hass)."""
         return 'keypad'
 
-    def handle_update(self, args):
+    def handle_update(self, args, _):
         """The callback invoked by a button's handle_update to
         set keypad value to the name of button.
 
@@ -2017,7 +2372,7 @@ class Task(VantageEntity):
         return 'Task name: "%s", vid: %d' % (
             self._name, self._vid)
 
-    def handle_update(self, args):
+    def handle_update(self, args, _):
         """Handle events from the task object.
 
         This callback is invoked from the VantageConnection thread.
@@ -2059,7 +2414,7 @@ class PollingSensor(VantageSensor):
             k = 'VARIABLE'
         self._vantage.send("GET"+k, self._vid)
 
-    def handle_update(self, args):
+    def handle_update(self, args, _):
         """Handle sensor updates.
 
         This callback is invoked from the VantageConnection thread.
@@ -2170,11 +2525,6 @@ class Shade(VantageEntity):
         self._vantage.register_id(Shade.CMD_TYPE, None, self)
         self._query_waiters = _RequestHelper()
 
-    @property
-    def kind(self):
-        """Returns the output type. At present AUTO_DETECT or NON_DIM."""
-        return self._load_type
-
     def __str__(self):
         """Returns pretty-printed representation of this object."""
         return 'Shade name: "%s", vid: %d, area: %d, level: %s' % (
@@ -2228,7 +2578,7 @@ class Shade(VantageEntity):
         """Stop the shade."""
         self._vantage.send("BLIND", self._vid, "CLOSE")
 
-    def handle_update(self, args):
+    def handle_update(self, args, _):
         """Handle new value for shade.
 
         This callback is invoked from the VantageConnection thread.
